"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.annotateSourceCode = exports.createTypesFile = exports.convertObjectToTypeDefinition = exports.usingFastify = exports.isNestMiddleware = exports.formatI18nErrors = exports.i18nValidationMessage = exports.i18nValidationErrorFactory = exports.shouldResolve = void 0;
const i18n_validation_error_interface_1 = require("../interfaces/i18n-validation-error.interface");
const ts = require("typescript");
const typescript_1 = require("typescript");
function shouldResolve(e) {
    return typeof e === 'function' || e['use'];
}
exports.shouldResolve = shouldResolve;
function validationErrorToI18n(e) {
    var _a;
    return {
        property: e.property,
        children: (_a = e === null || e === void 0 ? void 0 : e.children) === null || _a === void 0 ? void 0 : _a.map(validationErrorToI18n),
        constraints: !!e.constraints
            ? Object.keys(e.constraints).reduce((result, key) => {
                result[key] = e.constraints[key];
                return result;
            }, {})
            : {},
    };
}
function i18nValidationErrorFactory(errors) {
    return new i18n_validation_error_interface_1.I18nValidationException(errors.map((e) => {
        return validationErrorToI18n(e);
    }));
}
exports.i18nValidationErrorFactory = i18nValidationErrorFactory;
function i18nValidationMessage(key, args) {
    return (a) => {
        const { constraints } = a;
        let { value } = a;
        if (typeof value === 'string') {
            value = value.replace(/\|/g, '');
        }
        return `${key}|${JSON.stringify(Object.assign({ value, constraints }, args))}`;
    };
}
exports.i18nValidationMessage = i18nValidationMessage;
function formatI18nErrors(errors, i18n, options) {
    return errors.map((error) => {
        var _a;
        error.children = formatI18nErrors((_a = error.children) !== null && _a !== void 0 ? _a : [], i18n, options);
        error.constraints = Object.keys(error.constraints).reduce((result, key) => {
            const [translationKey, argsString] = error.constraints[key].split('|');
            const args = !!argsString ? JSON.parse(argsString) : {};
            result[key] = i18n.translate(translationKey, Object.assign(Object.assign({}, options), { args: Object.assign({ property: error.property }, args) }));
            return result;
        }, {});
        return error;
    });
}
exports.formatI18nErrors = formatI18nErrors;
const isNestMiddleware = (consumer) => {
    return typeof consumer.httpAdapter === 'object';
};
exports.isNestMiddleware = isNestMiddleware;
const usingFastify = (consumer) => {
    return consumer.httpAdapter.constructor.name
        .toLowerCase()
        .startsWith('fastify');
};
exports.usingFastify = usingFastify;
const convertObjectToTypeDefinition = (object) => {
    switch (typeof object) {
        case 'object':
            return Object.keys(object).map((key) => {
                if (typeof object[key] === 'string') {
                    return typescript_1.factory.createPropertySignature(undefined, typescript_1.factory.createStringLiteral(key), undefined, typescript_1.factory.createKeywordTypeNode(ts.SyntaxKind.StringKeyword));
                }
                if (Array.isArray(object[key])) {
                    return typescript_1.factory.createPropertySignature(undefined, typescript_1.factory.createStringLiteral(key), undefined, typescript_1.factory.createTupleTypeNode(Array(object[key].length).fill(typescript_1.factory.createKeywordTypeNode(ts.SyntaxKind.StringKeyword))));
                }
                return typescript_1.factory.createPropertySignature(undefined, typescript_1.factory.createStringLiteral(key), undefined, typescript_1.factory.createTypeLiteralNode((0, exports.convertObjectToTypeDefinition)(object[key])));
            });
    }
    return [];
};
exports.convertObjectToTypeDefinition = convertObjectToTypeDefinition;
const printer = ts.createPrinter();
const createTypesFile = (object) => {
    const sourceFile = ts.createSourceFile('placeholder.ts', '', ts.ScriptTarget.ESNext, true, ts.ScriptKind.TS);
    const i18nTranslationsType = typescript_1.factory.createTypeAliasDeclaration([typescript_1.factory.createModifier(ts.SyntaxKind.ExportKeyword)], typescript_1.factory.createIdentifier('I18nTranslations'), undefined, typescript_1.factory.createTypeLiteralNode((0, exports.convertObjectToTypeDefinition)(object)));
    const nodes = typescript_1.factory.createNodeArray([
        typescript_1.factory.createImportDeclaration(undefined, typescript_1.factory.createImportClause(false, undefined, typescript_1.factory.createNamedImports([
            typescript_1.factory.createImportSpecifier(false, undefined, typescript_1.factory.createIdentifier('Path')),
        ])), typescript_1.factory.createStringLiteral('nestjs-i18n'), undefined),
        i18nTranslationsType,
        typescript_1.factory.createTypeAliasDeclaration([typescript_1.factory.createModifier(ts.SyntaxKind.ExportKeyword)], typescript_1.factory.createIdentifier('I18nPath'), undefined, typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('Path'), [
            typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('I18nTranslations'), undefined),
        ])),
    ]);
    return printer.printList(ts.ListFormat.MultiLine, nodes, sourceFile);
};
exports.createTypesFile = createTypesFile;
const annotateSourceCode = (code) => {
    return `/* DO NOT EDIT, file generated by nestjs-i18n */

${code}`;
};
exports.annotateSourceCode = annotateSourceCode;
//# sourceMappingURL=util.js.map